@@ -1,49 +1,25 @@
 package org.kohsuke.args4j;
 
+import java.io.BufferedReader;
 import java.io.File;
+import java.io.FileReader;
+import java.io.IOException;
 import java.io.OutputStream;
 import java.io.OutputStreamWriter;
 import java.io.PrintWriter;
 import java.io.Writer;
-import java.lang.reflect.Constructor;
-import java.lang.reflect.InvocationTargetException;
-import java.net.InetAddress;
-import java.net.URI;
-import java.net.URL;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collection;
 import java.util.Collections;
-import java.util.Comparator;
-import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
-import java.util.Map;
 import java.util.ResourceBundle;
 import java.util.Set;
-import java.util.TreeMap;
-import java.util.Collection;
-import java.util.logging.Logger;
 
-import org.kohsuke.args4j.spi.BooleanOptionHandler;
-import org.kohsuke.args4j.spi.ByteOptionHandler;
-import org.kohsuke.args4j.spi.CharOptionHandler;
-import org.kohsuke.args4j.spi.DoubleOptionHandler;
-import org.kohsuke.args4j.spi.EnumOptionHandler;
-import org.kohsuke.args4j.spi.FileOptionHandler;
-import org.kohsuke.args4j.spi.PathOptionHandler;
-import org.kohsuke.args4j.spi.FloatOptionHandler;
-import org.kohsuke.args4j.spi.InetAddressOptionHandler;
-import org.kohsuke.args4j.spi.IntOptionHandler;
-import org.kohsuke.args4j.spi.LongOptionHandler;
-import org.kohsuke.args4j.spi.MapOptionHandler;
 import org.kohsuke.args4j.spi.OptionHandler;
 import org.kohsuke.args4j.spi.Parameters;
 import org.kohsuke.args4j.spi.Setter;
-import org.kohsuke.args4j.spi.ShortOptionHandler;
-import org.kohsuke.args4j.spi.StringOptionHandler;
-import org.kohsuke.args4j.spi.URIOptionHandler;
-import org.kohsuke.args4j.spi.URLOptionHandler;
-
 
 /**
  * Command line argument owner.
@@ -55,6 +31,7 @@ import org.kohsuke.args4j.spi.URLOptionHandler;
  *     Kohsuke Kawaguchi (kk@kohsuke.org)
  */
 public class CmdLineParser {
+
     /**
      * Discovered {@link OptionHandler}s for options.
      */
@@ -69,12 +46,9 @@ public class CmdLineParser {
     private OptionHandler currentOptionHandler = null;
 
 	/**
-	 *  The length of a usage line.
-	 *  If the usage message is longer than this value, the parser wraps the line.
-     *  
-     *  Defaults to {@code 80}.
+     * settings for the parser
 	 */
-	private int usageWidth = 80;
+	private ParserProperties parserProperties;
 
     /**
      * Creates a new command line owner that
@@ -90,6 +64,27 @@ public class CmdLineParser {
      *      if the option bean class is using args4j annotations incorrectly.
      */
     public CmdLineParser(Object bean) {
+        // for display purposes, we like the arguments in argument order, but the options in alphabetical order
+        this(bean, ParserProperties.defaults());
+    }
+
+    /**
+     * Creates a new command line owner that
+     * parses arguments/options and set them into
+     * the given object.
+     *
+     * @param bean
+     *      instance of a class annotated by {@link Option} and {@link Argument}.
+     *      this object will receive values. If this is {@code null}, the processing will
+     *      be skipped, which is useful if you'd like to feed metadata from other sources.
+     *
+     * @param parserProperties various settings for this class
+     *
+     * @throws IllegalAnnotationError
+     *      if the option bean class is using args4j annotations incorrectly.
+     */
+    public CmdLineParser(Object bean, ParserProperties parserProperties) {
+        this.parserProperties = parserProperties;
         // A 'return' in the constructor just skips the rest of the implementation
         // and returns the new object directly.
         if (bean==null) return;
@@ -97,12 +92,27 @@ public class CmdLineParser {
         // Parse the metadata and create the setters
         new ClassParser().parse(bean,this);
 
-        // for display purposes, we like the arguments in argument order, but the options in alphabetical order
-        Collections.sort(options, new Comparator<OptionHandler>() {
-			public int compare(OptionHandler o1, OptionHandler o2) {
-				return o1.option.toString().compareTo(o2.option.toString());
-			}
-		});
+        if (parserProperties.getOptionSorter()!=null) {
+            Collections.sort(options, parserProperties.getOptionSorter());
+        }
+    }
+
+    public ParserProperties getProperties() {
+        return parserProperties;
+    }
+
+    /** This method is similar to {@code Objects.requireNonNull()}.
+     * But this one is available for JDK 1.6 which is the
+     * current target of args4j.
+     * I didn't want to break compatibility with JDK 1.6.
+     * @param obj the object to check for {@code null} value.
+     * @param name the object name. If {@code obj} is {@code null}, then
+     * an exception is constructed from this name.
+     */
+    private static void checkNonNull(Object obj, String name) {
+        if (obj == null) {
+            throw new NullPointerException(name+" is null");
+        }
     }
 
     /**
@@ -110,9 +120,14 @@ public class CmdLineParser {
      *
      * @param setter the setter for the type
      * @param a the Argument
+     * @throws NullPointerException if {@code setter} or {@code a} is {@code null}.
      */
     public void addArgument(Setter setter, Argument a) {
-        OptionHandler h = createOptionHandler(new OptionDef(a,setter.isMultiValued()),setter);
+        Utilities.checkNonNull(setter, "Setter");
+        Utilities.checkNonNull(a, "Argument");
+        
+        OptionHandler h = OptionHandlerRegistry.getRegistry().createOptionHandler(this,
+                new OptionDef(a,setter.isMultiValued()),setter);
     	int index = a.index();
     	// make sure the argument will fit in the list
     	while (index >= arguments.size()) {
@@ -121,7 +136,7 @@ public class CmdLineParser {
     	if(arguments.get(index)!=null) {
             throw new IllegalAnnotationError(Messages.MULTIPLE_USE_OF_ARGUMENT.format(index));
         }
-    	arguments.set(index,h);
+    	arguments.set(index, h);
     }
 
     /**
@@ -129,13 +144,19 @@ public class CmdLineParser {
      *
      * @param setter the setter for the type
      * @param o the {@code Option}
+     * @throws NullPointerException if {@code setter} or {@code o} is {@code null}.
+     * @throws IllegalAnnotationError if the option name or one of the aliases is already taken.
      */
     public void addOption(Setter setter, Option o) {
+        checkNonNull(setter, "Setter");
+        checkNonNull(o, "Option");
+    
         checkOptionNotInMap(o.name());
         for (String alias : o.aliases()) {
         	checkOptionNotInMap(alias);
         }
-        options.add(createOptionHandler(new NamedOptionDef(o), setter));
+        options.add(OptionHandlerRegistry.getRegistry().createOptionHandler(
+                this, new NamedOptionDef(o), setter));
     }
 
     /**
@@ -153,6 +174,8 @@ public class CmdLineParser {
     }
 
 	private void checkOptionNotInMap(String name) throws IllegalAnnotationError {
+        checkNonNull(name, "name");
+        
 		if(findOptionByName(name)!=null) {
             throw new IllegalAnnotationError(Messages.MULTIPLE_USE_OF_OPTION.format(name));
         }
@@ -161,37 +184,12 @@ public class CmdLineParser {
     /**
      * Creates an {@link OptionHandler} that handles the given {@link Option} annotation
      * and the {@link Setter} instance.
+     * @deprecated You should use {@link OptionHandlerRegistry#createOptionHandler(org.kohsuke.args4j.CmdLineParser, org.kohsuke.args4j.OptionDef, org.kohsuke.args4j.spi.Setter) } instead.
      */
-   @SuppressWarnings("unchecked")
     protected OptionHandler createOptionHandler(OptionDef o, Setter setter) {
-
-        Constructor<? extends OptionHandler> handlerType;
-        Class<? extends OptionHandler> h = o.handler();
-
-        if(h==OptionHandler.class) {
-            // infer the type
-
-            // enum is the special case
-            Class t = setter.getType();
-            if(Enum.class.isAssignableFrom(t))
-                return new EnumOptionHandler(this,o,setter,t);
-
-            handlerType = handlerClasses.get(t);
-            if(handlerType==null)
-                throw new IllegalAnnotationError(Messages.UNKNOWN_HANDLER.format(t));
-        } else {
-            handlerType = getConstructor(h);
-        }
-
-        try {
-            return handlerType.newInstance(this,o,setter);
-        } catch (InstantiationException e) {
-            throw new IllegalAnnotationError(e);
-        } catch (IllegalAccessException e) {
-            throw new IllegalAnnotationError(e);
-        } catch (InvocationTargetException e) {
-            throw new IllegalAnnotationError(e);
-        }
+        checkNonNull(o, "OptionDef");
+        checkNonNull(setter, "Setter");
+        return OptionHandlerRegistry.getRegistry().createOptionHandler(this, o, setter);
     }
 
     /**
@@ -213,14 +211,14 @@ public class CmdLineParser {
      *      Use {@link #printExample(OptionHandlerFilter)}
      */
     public String printExample(ExampleMode mode) {
-        return printExample(mode,null);
+        return printExample(mode, null);
     }
 
     /**
      * Formats a command line example into a string.
      *
      * <p>
-     * This method produces a string like <samp> -d &lt;dir> -v -b</samp>.
+     * This method produces a string like <code> -d &lt;dir&gt; -v -b</code>.
      * This is useful for printing a command line example (perhaps
      * as a part of the usage screen).
      *
@@ -229,7 +227,7 @@ public class CmdLineParser {
      *      Determines which options will be a part of the returned string.
      *      Must not be {@code null}.
      * @param rb
-     *      If non-{@code null}, meta variables (<samp>&lt;dir></samp> in the above example)
+     *      If non-{@code null}, meta variables (<code>&lt;dir&gt;</code> in the above example)
      *      is treated as a key to this resource bundle, and the associated
      *      value is printed. See {@link Option#metaVar()}. This is to support
      *      localization.
@@ -240,18 +238,21 @@ public class CmdLineParser {
      *      just the empty string {@code ""}. Otherwise, this method returns a
      *      string that contains a space at the beginning (but not at the end).
      *      This allows you to do something like:
-     *      <code><pre>System.err.println("java -jar my.jar"+parser.printExample(REQUIRED)+" arg1 arg2");</pre></code>
+     *      <code>System.err.println("java -jar my.jar"+parser.printExample(REQUIRED)+" arg1 arg2");</code>
+     * @throws NullPointerException if {@code mode} is {@code null}.
      */
-    public String printExample(OptionHandlerFilter mode,ResourceBundle rb) {
+    public String printExample(OptionHandlerFilter mode, ResourceBundle rb) {
         StringBuilder buf = new StringBuilder();
 
+        Utilities.checkNonNull(mode, "mode");
+        
         for (OptionHandler h : options) {
             OptionDef option = h.option;
             if(option.usage().length()==0)  continue;   // ignore
             if(!mode.select(h))             continue;
 
             buf.append(' ');
-            buf.append(h.getNameAndMeta(rb));
+            buf.append(h.getNameAndMeta(rb, parserProperties));
         }
 
         return buf.toString();
@@ -261,8 +262,8 @@ public class CmdLineParser {
      * @deprecated
      *      Use {@link #printExample(OptionHandlerFilter,ResourceBundle)}
      */
-    public String printExample(ExampleMode mode,ResourceBundle rb) {
-        return printExample((OptionHandlerFilter)mode,rb);
+    public String printExample(ExampleMode mode, ResourceBundle rb) {
+        return printExample((OptionHandlerFilter) mode, rb);
     }
 
     /**
@@ -283,7 +284,7 @@ public class CmdLineParser {
      * Short for {@code printUsage(out,rb,OptionHandlerFilter.PUBLIC)}
      */
     public void printUsage(Writer out, ResourceBundle rb) {
-        printUsage(out,rb,OptionHandlerFilter.PUBLIC);
+        printUsage(out, rb, OptionHandlerFilter.PUBLIC);
     }
 
     /**
@@ -341,18 +342,19 @@ public class CmdLineParser {
     	}
 
     	// What is the width of the two data columns
-    	int widthMetadata = Math.min(len, (usageWidth - 4) / 2);
-    	int widthUsage    = usageWidth - 4 - widthMetadata;
+        int totalUsageWidth = parserProperties.getUsageWidth();
+    	int widthMetadata = Math.min(len, (totalUsageWidth - 4) / 2);
+    	int widthUsage    = totalUsageWidth - 4 - widthMetadata;
 
     	// Line wrapping
-    	List<String> namesAndMetas = wrapLines(handler.getNameAndMeta(rb), widthMetadata);
+    	List<String> namesAndMetas = wrapLines(handler.getNameAndMeta(rb, parserProperties), widthMetadata);
     	List<String> usages        = wrapLines(localize(handler.option.usage(),rb), widthUsage);
 
     	// Output
     	for(int i=0; i<Math.max(namesAndMetas.size(), usages.size()); i++) {
     		String nameAndMeta = (i >= namesAndMetas.size()) ? "" : namesAndMetas.get(i);
 			String usage       = (i >= usages.size())        ? "" : usages.get(i);
-			String format      = (nameAndMeta.length() > 0)
+			String format      = ((nameAndMeta.length() > 0) && (i == 0))
 			                   ? " %1$-" + widthMetadata + "s : %2$-1s"
 			                   : " %1$-" + widthMetadata + "s   %2$-1s";
 			String output = String.format(format, nameAndMeta, usage);
@@ -380,7 +382,7 @@ public class CmdLineParser {
                 int lineLength;
                 String candidate = restOfLine.substring(0, maxLength);
                 int sp=candidate.lastIndexOf(' ');
-                if(sp>maxLength*3/4)    lineLength=sp;
+                if(sp>maxLength*3/5)    lineLength=sp;
                 else                    lineLength=maxLength;
                 rv.add(restOfLine.substring(0, lineLength));
                 restOfLine = restOfLine.substring(lineLength).trim();
@@ -394,7 +396,7 @@ public class CmdLineParser {
 		if(h.option.usage().length()==0)
 			return 0;
 
-		return h.getNameAndMeta(rb).length();
+		return h.getNameAndMeta(rb, parserProperties).length();
 	}
 
     /**
@@ -424,7 +426,7 @@ public class CmdLineParser {
 
         public String getParameter(int idx) throws CmdLineException {
 			if( pos+idx>=args.length || pos+idx<0 )
-                throw new CmdLineException(CmdLineParser.this, Messages.MISSING_OPERAND.format(getOptionName()));
+                throw new CmdLineException(CmdLineParser.this, Messages.MISSING_OPERAND, getOptionName());
             return args[pos+idx];
         }
 
@@ -451,7 +453,7 @@ public class CmdLineParser {
     }
 
     /**
-     * Same as {@link #parseArgument(String[])} 
+     * Same as {@link #parseArgument(String[])}
      */
     public void parseArgument(Collection<String> args) throws CmdLineException {
         parseArgument(args.toArray(new String[args.size()]));
@@ -466,9 +468,17 @@ public class CmdLineParser {
      * @throws CmdLineException
      *      if there's any error parsing arguments, or if
      *      {@link Option#required() required} option was not given.
+     * @throws NullPointerException if {@code args} is {@code null}.
      */
     public void parseArgument(final String... args) throws CmdLineException {
-        CmdLineImpl cmdLine = new CmdLineImpl(args);
+        
+        Utilities.checkNonNull(args, "args");
+        
+        String expandedArgs[] = args;
+        if (parserProperties.getAtSyntax()) {
+            expandedArgs = expandAtFiles(args);
+        }
+        CmdLineImpl cmdLine = new CmdLineImpl(expandedArgs);
 
         Set<OptionHandler> present = new HashSet<OptionHandler>();
         int argIndex = 0;
@@ -476,13 +486,15 @@ public class CmdLineParser {
         while( cmdLine.hasMore() ) {
             String arg = cmdLine.getCurrentToken();
             if( isOption(arg) ) {
-            	boolean isKeyValuePair = arg.indexOf('=')!=-1;
+                // '=' is for historical compatibility fallback
+                boolean isKeyValuePair = arg.contains(parserProperties.getOptionValueDelimiter()) || arg.indexOf('=')!=-1;
+
                 // parse this as an option.
                 currentOptionHandler = isKeyValuePair ? findOptionHandler(arg) : findOptionByName(arg);
 
                 if(currentOptionHandler==null) {
                     // TODO: insert dynamic handler processing
-                    throw new CmdLineException(this, Messages.UNDEFINED_OPTION.format(arg));
+                    throw new CmdLineException(this, Messages.UNDEFINED_OPTION, arg);
                 }
 
                 // known option; skip its name
@@ -494,7 +506,7 @@ public class CmdLineParser {
             } else {
             	if (argIndex >= arguments.size()) {
             		Messages msg = arguments.size() == 0 ? Messages.NO_ARGUMENT_ALLOWED : Messages.TOO_MANY_ARGUMENTS;
-                    throw new CmdLineException(this, msg.format(arg));
+                    throw new CmdLineException(this, msg, arg);
             	}
 
             	// known argument
@@ -510,54 +522,130 @@ public class CmdLineParser {
         	present.add(currentOptionHandler);
         }
 
+        // check whether a help option is set
+        boolean helpSet = false;
+        for (OptionHandler handler : options) {
+            if(handler.option.help() && present.contains(handler)) {
+                helpSet = true;
+            }
+        }
+
+        if (!helpSet) {
+            checkRequiredOptionsAndArguments(present);
+        }
+    }
+    
+    /**
+     * Expands every entry prefixed with the AT sign by
+     * reading the file. The AT sign is used to reference
+     * another file that contains command line options separated
+     * by line breaks. 
+     * @param args the command line arguments to be preprocessed.
+     * @return args with the @ sequences replaced by the text files referenced
+     * by the @ sequences, split around the line breaks.
+     * @throws CmdLineException 
+     */
+    private String[] expandAtFiles(String args[]) throws CmdLineException {
+        List<String> result = new ArrayList<String>();
+        for (String arg : args) {
+            if (arg.startsWith("@")) {
+                File file = new File(arg.substring(1));
+                if (!file.exists())
+                    throw new CmdLineException(this,Messages.NO_SUCH_FILE,file.getPath());
+                try {
+                    result.addAll(readAllLines(file));
+                } catch (IOException ex) {
+                    throw new CmdLineException(this, "Failed to parse "+file,ex);
+                }
+            } else {
+                result.add(arg);
+            }
+        }
+        return result.toArray(new String[result.size()]);
+    }
+    
+    /**
+     * Reads all lines of a file with the platform encoding.
+     */
+    private static List<String> readAllLines(File f) throws IOException {
+        BufferedReader r = new BufferedReader(new FileReader(f));
+        try {
+            List<String> result = new ArrayList<String>();
+            String line;
+            while ((line = r.readLine()) != null) {
+                result.add(line);
+            }
+            return result;
+        }  finally {
+            r.close();
+        }
+    }
+
+    private void checkRequiredOptionsAndArguments(Set<OptionHandler> present) throws CmdLineException {
         // make sure that all mandatory options are present
-        for (OptionHandler handler : options)
-            if(handler.option.required() && !present.contains(handler))
-                throw new CmdLineException(this, Messages.REQUIRED_OPTION_MISSING.format(handler.option.toString()));
+        for (OptionHandler handler : options) {
+            if(handler.option.required() && !present.contains(handler)) {
+                throw new CmdLineException(this, Messages.REQUIRED_OPTION_MISSING, handler.option.toString());
+            }
+        }
 
         // make sure that all mandatory arguments are present
-        for (OptionHandler handler : arguments)
-            if(handler.option.required() && !present.contains(handler))
-                throw new CmdLineException(this, Messages.REQUIRED_ARGUMENT_MISSING.format(handler.option.toString()));
+        for (OptionHandler handler : arguments) {
+            if(handler.option.required() && !present.contains(handler)) {
+                throw new CmdLineException(this, Messages.REQUIRED_ARGUMENT_MISSING, handler.option.toString());
+            }
+        }
 
         //make sure that all requires arguments are present
-        for(OptionHandler handler : present) {
-            if(handler.option instanceof NamedOptionDef && !isHandlerHasHisOptions((NamedOptionDef)handler.option, present)) {
-                throw new CmdLineException(this, Messages.REQUIRES_OPTION_MISSING
-                        .format(handler.option.toString(), Arrays.toString(((NamedOptionDef)handler.option).depends())));
+        for (OptionHandler handler : present) {
+            if (handler.option instanceof NamedOptionDef && !isHandlerHasHisOptions((NamedOptionDef)handler.option, present)) {
+                throw new CmdLineException(this, Messages.REQUIRES_OPTION_MISSING,
+                        handler.option.toString(), Arrays.toString(((NamedOptionDef)handler.option).depends()));
+            }
+        }
+        
+        //make sure that all forbids arguments are not present
+        for (OptionHandler handler : present) {
+            if (handler.option instanceof NamedOptionDef && !isHandlerAllowOtherOptions((NamedOptionDef) handler.option, present)) {
+                throw new CmdLineException(this, Messages.FORBIDDEN_OPTION_PRESENT,
+                        handler.option.toString(), Arrays.toString(((NamedOptionDef) handler.option).forbids()));
             }
         }
     }
 
     /**
-     * @param option
-     * @param present
      * @return {@code true} if all options required by {@code option} are present, {@code false} otherwise
      */
     private boolean isHandlerHasHisOptions(NamedOptionDef option, Set<OptionHandler> present) {
-        if (option.depends() != null) {
-            for (String depend : option.depends()) {
-                if (!present.contains(findOptionHandler(depend)))
-                    return false;
-            }
+        for (String depend : option.depends()) {
+            if (!present.contains(findOptionHandler(depend)))
+                return false;
         }
         return true;
     }
 
+    /**
+     * @return {@code true} if all options forbid by {@code option} are not present, {@code false} otherwise
+     */
+    private boolean isHandlerAllowOtherOptions(NamedOptionDef option, Set<OptionHandler> present) {
+        for (String forbid : option.forbids()) {
+            if (present.contains(findOptionHandler(forbid)))
+                return false;
+        }
+        return true;
+    }
+    
     private OptionHandler findOptionHandler(String name) {
-		OptionHandler handler = findOptionByName(name);
-		if (handler==null) {
-			// Have not found by its name, maybe its a property?
-			// Search for parts of the name (=prefix) - most specific first
-			for (int i=name.length(); i>1; i--) {
-				String prefix = name.substring(0, i);
-				Map<String,OptionHandler> possibleHandlers = filter(options, prefix);
-				handler = possibleHandlers.get(prefix);
-				if (handler!=null) return handler;
-			}
-		}
-		return handler;
-	}
+        // Look for key/value pair first.
+        int pos = name.indexOf(parserProperties.getOptionValueDelimiter());
+        if (pos < 0) {
+            pos = name.indexOf('=');    // historical compatibility fallback
+        }
+        if (pos > 0) {
+            name = name.substring(0, pos);
+        }
+		return findOptionByName(name);
+    }
 
 	/**
 	 * Finds a registered {@code OptionHandler} by its name or its alias.
@@ -579,44 +667,17 @@ public class CmdLineParser {
 		return null;
 	}
 
-
-  private Map<String,OptionHandler> filter(List<OptionHandler> opt, String keyFilter) {
-    Map<String,OptionHandler> rv = new TreeMap<String,OptionHandler>();
-    for (OptionHandler h : opt) {
-      NamedOptionDef option = (NamedOptionDef)h.option;
-      String prefix = "";
-      for (String alias : option.aliases()) {
-        if (keyFilter.startsWith(alias)) {
-          prefix = keyFilter;
-          break;
-        }
-      }
-      if (option.name().startsWith(keyFilter)){
-        prefix = keyFilter;
-      }
-      rv.put(prefix, h);
-    }
-    return rv;
-  }
-
-
     /**
      * Returns {@code true} if the given token is an option
      * (as opposed to an argument).
+     * @throws NullPointerException if {@code arg} is {@code null}.
      */
     protected boolean isOption(String arg) {
+        Utilities.checkNonNull(arg, "arg");
+        
         return parsingOptions && arg.startsWith("-");
     }
 
-
-    /**
-     * All {@link OptionHandler}s known to the {@link CmdLineParser}.
-     *
-     * Constructors of {@link OptionHandler}-derived class keyed by their supported types.
-     */
-    private static final Map<Class,Constructor<? extends OptionHandler>> handlerClasses =
-            Collections.synchronizedMap(new HashMap<Class,Constructor<? extends OptionHandler>>());
-
     /**
      * Registers a user-defined {@link OptionHandler} class with args4j.
      *
@@ -630,62 +691,35 @@ public class CmdLineParser {
      * @param handlerClass
      *      This class must have the constructor that has the same signature as
      *      {@link OptionHandler#OptionHandler(CmdLineParser, OptionDef, Setter)}
+     * @throws NullPointerException if {@code valueType} or {@code handlerClass} is {@code null}.
+     * @throws IllegalArgumentException if {@code handlerClass} is not a subtype of {@code OptionHandler}.
+     * @deprecated You should use {@link OptionHandlerRegistry#registerHandler(java.lang.Class, java.lang.Class)} instead.
      */
     public static void registerHandler( Class valueType, Class<? extends OptionHandler> handlerClass ) {
-        if(valueType==null || handlerClass==null)
-            throw new IllegalArgumentException();
-        if(!OptionHandler.class.isAssignableFrom(handlerClass))
-            throw new IllegalArgumentException(Messages.NO_OPTIONHANDLER.format());
+        checkNonNull(valueType, "valueType");
+        checkNonNull(handlerClass, "handlerClass");
 
-        Constructor<? extends OptionHandler> c = getConstructor(handlerClass);
-        handlerClasses.put(valueType,c);
-    }
-
-    private static Constructor<? extends OptionHandler> getConstructor(Class<? extends OptionHandler> handlerClass) {
-        try {
-            return handlerClass.getConstructor(CmdLineParser.class, OptionDef.class, Setter.class);
-        } catch (NoSuchMethodException e) {
-            throw new IllegalArgumentException(Messages.NO_CONSTRUCTOR_ON_HANDLER.format(handlerClass));
-        }
-    }
-
-    static {
-        registerHandler(Boolean.class,BooleanOptionHandler.class);
-        registerHandler(boolean.class,BooleanOptionHandler.class);
-        registerHandler(File.class,FileOptionHandler.class);
-        registerHandler(URL.class, URLOptionHandler.class);
-        registerHandler(URI.class, URIOptionHandler.class);
-        registerHandler(Integer.class,IntOptionHandler.class);
-        registerHandler(int.class,IntOptionHandler.class);
-        registerHandler(Double.class, DoubleOptionHandler.class);
-        registerHandler(double.class,DoubleOptionHandler.class);
-        registerHandler(String.class,StringOptionHandler.class);
-        registerHandler(Byte.class, ByteOptionHandler.class);
-        registerHandler(byte.class, ByteOptionHandler.class);
-        registerHandler(Character.class, CharOptionHandler.class);
-        registerHandler(char.class, CharOptionHandler.class);
-        registerHandler(Float.class, FloatOptionHandler.class);
-        registerHandler(float.class, FloatOptionHandler.class);
-        registerHandler(Long.class, LongOptionHandler.class);
-        registerHandler(long.class, LongOptionHandler.class);
-        registerHandler(Short.class, ShortOptionHandler.class);
-        registerHandler(short.class, ShortOptionHandler.class);
-        registerHandler(InetAddress.class, InetAddressOptionHandler.class);
-        // enum is a special case
-        registerHandler(Map.class,MapOptionHandler.class);
-
-        try {
-            Class p = Class.forName("java.nio.file.Path");
-            registerHandler(p, PathOptionHandler.class);
-        } catch (ClassNotFoundException e) {
-            // running in Java6 or earlier
-        }
+        OptionHandlerRegistry.getRegistry().registerHandler(valueType, handlerClass);
     }
 
+    /**
+     * Sets the width of the usage output.
+     * @param usageWidth the width of the usage output in columns.
+     * @throws IllegalArgumentException if {@code usageWidth} is negative
+     * @deprecated
+     *      Use {@link ParserProperties#withUsageWidth(int)} instead.
+     */
 	public void setUsageWidth(int usageWidth) {
-		this.usageWidth = usageWidth;
+        parserProperties.withUsageWidth(usageWidth);
 	}
 
+    /**
+     * Signals the parser that parsing the options has finished.
+     * 
+     * <p>
+     * Everything seen after this call is treated as an argument
+     * as opposed to an option.
+     */
 	public void stopOptionParsing() {
 		parsingOptions = false;
 	}
@@ -696,9 +730,12 @@ public class CmdLineParser {
      * <p>
      * This is a convenience method for calling {@code printUsage(new OutputStreamWriter(out),null)}
      * so that you can do {@code printUsage(System.err)}.
+     * @throws NullPointerException if {@code out} is {@code null}.
      */
 	public void printSingleLineUsage(OutputStream out) {
-		printSingleLineUsage(new OutputStreamWriter(out),null);
+        checkNonNull(out, "OutputStream");
+        
+		printSingleLineUsage(new OutputStreamWriter(out), null);
 	}
 
     /**
@@ -707,8 +744,12 @@ public class CmdLineParser {
      * @param rb
      *      if this is non-{@code null}, {@link Option#usage()} is treated
      *      as a key to obtain the actual message from this resource bundle.
+     * @throws NullPointerException if {@code w} is {@code null}.
      */
+    // TODO test this!
 	public void printSingleLineUsage(Writer w, ResourceBundle rb) {
+        Utilities.checkNonNull(w, "Writer");
+        
 		PrintWriter pw = new PrintWriter(w);
 		for (OptionHandler h : arguments) {
 			printSingleLineOption(pw, h, rb);
@@ -723,13 +764,11 @@ public class CmdLineParser {
 		pw.print(' ');
 		if (!h.option.required())
 			pw.print('[');
-		pw.print(h.getNameAndMeta(rb));
+		pw.print(h.getNameAndMeta(rb, parserProperties));
 		if (h.option.isMultiValued()) {
 			pw.print(" ...");
 		}
 		if (!h.option.required())
 			pw.print(']');
 	}
-
-    private static final Logger LOGGER = Logger.getLogger(CmdLineParser.class.getName());
 }
